import { expect, tap } from '@git.zone/tstest/tapbundle';
import { EInvoice } from '../../../ts/index.js';
import { PerformanceTracker } from '../performance.tracker.js';

tap.test('ENC-06: Namespace Declarations - should handle XML namespace declarations correctly', async () => {
  // ENC-06: Verify handling of Namespace Declarations encoded documents
  
  // Test 1: Direct Namespace Declarations encoding (expected to fail)
  console.log('\nTest 1: Direct Namespace Declarations encoding');
  const { result: directResult, metric: directMetric } = await PerformanceTracker.track(
    'namespace-direct',
    async () => {
      // XML parsers typically don't support Namespace Declarations directly
      const xmlContent = `<?xml version="1.0" encoding="Namespace Declarations"?>
<Invoice xmlns="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2">
  <UBLVersionID>2.1</UBLVersionID>
  <ID>NAMESPACE-TEST</ID>
  <IssueDate>2025-01-25</IssueDate>
  <DocumentCurrencyCode>EUR</DocumentCurrencyCode>
</Invoice>`;
      
      let success = false;
      let error = null;
      
      try {
        const newInvoice = new EInvoice();
        await newInvoice.fromXmlString(xmlContent);
        success = newInvoice.id === 'NAMESPACE-TEST' || 
                 newInvoice.invoiceId === 'NAMESPACE-TEST' ||
                 newInvoice.accountingDocId === 'NAMESPACE-TEST';
      } catch (e) {
        error = e;
        console.log(`  Namespace Declarations not directly supported: ${e.message}`);
      }
      
      return { success, error };
    }
  );
  
  console.log(`  Namespace Declarations direct test completed in ${directMetric.duration}ms`);
  
  // Test 2: UTF-8 fallback (should always work)
  console.log('\nTest 2: UTF-8 fallback');
  const { result: fallbackResult, metric: fallbackMetric } = await PerformanceTracker.track(
    'namespace-fallback',
    async () => {
      const einvoice = new EInvoice();
      einvoice.id = 'NAMESPACE-FALLBACK-TEST';
      einvoice.issueDate = new Date(2025, 0, 25);
      einvoice.invoiceId = 'NAMESPACE-FALLBACK-TEST';
      einvoice.accountingDocId = 'NAMESPACE-FALLBACK-TEST';
      einvoice.subject = 'Namespace Declarations fallback test';
      
      einvoice.from = {
        type: 'company',
        name: 'Test Company',
        description: 'Testing Namespace Declarations encoding',
        address: {
          streetName: 'Test Street',
          houseNumber: '1',
          postalCode: '12345',
          city: 'Test City',
          country: 'DE'
        },
        status: 'active',
        foundedDate: { year: 2020, month: 1, day: 1 },
        registrationDetails: {
          vatId: 'DE123456789',
          registrationId: 'HRB 12345',
          registrationName: 'Commercial Register'
        }
      };
      
      einvoice.to = {
        type: 'person',
        name: 'Test',
        surname: 'Customer',
        salutation: 'Mr' as const,
        sex: 'male' as const,
        title: 'Doctor' as const,
        description: 'Test customer',
        address: {
          streetName: 'Customer Street',
          houseNumber: '2',
          postalCode: '54321',
          city: 'Customer City',
          country: 'DE'
        }
      };
      
      einvoice.items = [{
        position: 1,
        name: 'Test Product',
        articleNumber: 'NAMESPACE-001',
        unitType: 'EA',
        unitQuantity: 1,
        unitNetPrice: 100,
        vatPercentage: 19
      }];
      
      // Export as UTF-8 (our default)
      const utf8Xml = await einvoice.toXmlString('ubl');
      
      // Verify UTF-8 works correctly
      const newInvoice = new EInvoice();
      await newInvoice.fromXmlString(utf8Xml);
      
      const success = newInvoice.id === 'NAMESPACE-FALLBACK-TEST' || 
                     newInvoice.invoiceId === 'NAMESPACE-FALLBACK-TEST' ||
                     newInvoice.accountingDocId === 'NAMESPACE-FALLBACK-TEST';
      
      console.log(`  UTF-8 fallback works: ${success}`);
      
      return { success };
    }
  );
  
  console.log(`  Namespace Declarations fallback test completed in ${fallbackMetric.duration}ms`);
  
  // Summary
  console.log('\n=== Namespace Declarations Encoding Test Summary ===');
  console.log(`Namespace Declarations Direct: ${directResult.success ? 'Supported' : 'Not supported (acceptable)'}`);
  console.log(`UTF-8 Fallback: ${fallbackResult.success ? 'Working' : 'Failed'}`);
  
  // The test passes if UTF-8 fallback works, since Namespace Declarations support is optional
  expect(fallbackResult.success).toBeTrue();
});

// Run the test
tap.start();
