import { tap } from '@git.zone/tstest/tapbundle';
import * as plugins from '../plugins.js';
import { EInvoice } from '../../../ts/index.js';
import { PerformanceTracker } from '../performance.tracker.js';
import * as os from 'os';

const performanceTracker = new PerformanceTracker('SEC-10: Resource Limits');

tap.test('SEC-10: Resource Limits - should enforce resource consumption limits', async (t) => {
  const einvoice = new EInvoice();

  // Test 1: File size limits
  const fileSizeLimits = await performanceTracker.measureAsync(
    'file-size-limits',
    async () => {
      const testSizes = [
        { size: 1 * 1024 * 1024, name: '1MB', shouldPass: true },
        { size: 10 * 1024 * 1024, name: '10MB', shouldPass: true },
        { size: 50 * 1024 * 1024, name: '50MB', shouldPass: true },
        { size: 100 * 1024 * 1024, name: '100MB', shouldPass: false },
        { size: 500 * 1024 * 1024, name: '500MB', shouldPass: false }
      ];
      
      const results = [];
      
      for (const test of testSizes) {
        // Create large XML content
        const chunk = '<Item>'.padEnd(1024, 'X') + '</Item>'; // ~1KB per item
        const itemCount = Math.floor(test.size / 1024);
        let largeXML = '<?xml version="1.0" encoding="UTF-8"?><Invoice><Items>';
        
        // Build in chunks to avoid memory issues
        for (let i = 0; i < itemCount; i += 1000) {
          const batchSize = Math.min(1000, itemCount - i);
          largeXML += chunk.repeat(batchSize);
        }
        largeXML += '</Items></Invoice>';
        
        try {
          const startTime = Date.now();
          const result = await einvoice.parseXML(largeXML, { maxSize: 50 * 1024 * 1024 });
          const timeTaken = Date.now() - startTime;
          
          results.push({
            size: test.name,
            passed: true,
            expectedPass: test.shouldPass,
            timeTaken,
            actualSize: largeXML.length
          });
        } catch (error) {
          results.push({
            size: test.name,
            passed: false,
            expectedPass: test.shouldPass,
            error: error.message,
            actualSize: largeXML.length
          });
        }
      }
      
      return results;
    }
  );

  fileSizeLimits.forEach(result => {
    if (result.expectedPass) {
      t.ok(result.passed, `File size ${result.size} should be accepted`);
    } else {
      t.notOk(result.passed, `File size ${result.size} should be rejected`);
    }
  });

  // Test 2: Memory usage limits
  const memoryUsageLimits = await performanceTracker.measureAsync(
    'memory-usage-limits',
    async () => {
      const baselineMemory = process.memoryUsage().heapUsed;
      const maxMemoryIncrease = 200 * 1024 * 1024; // 200MB limit
      
      const operations = [
        {
          name: 'large-attribute-count',
          fn: async () => {
            let attrs = '';
            for (let i = 0; i < 1000000; i++) {
              attrs += ` attr${i}="value"`;
            }
            return `<Invoice ${attrs}></Invoice>`;
          }
        },
        {
          name: 'deep-nesting',
          fn: async () => {
            let xml = '';
            for (let i = 0; i < 10000; i++) {
              xml += `<Level${i}>`;
            }
            xml += 'data';
            for (let i = 9999; i >= 0; i--) {
              xml += `</Level${i}>`;
            }
            return xml;
          }
        },
        {
          name: 'large-text-nodes',
          fn: async () => {
            const largeText = 'A'.repeat(50 * 1024 * 1024); // 50MB
            return `<Invoice><Description>${largeText}</Description></Invoice>`;
          }
        }
      ];
      
      const results = [];
      
      for (const op of operations) {
        try {
          const xml = await op.fn();
          const startMemory = process.memoryUsage().heapUsed;
          
          await einvoice.parseXML(xml, { maxMemory: maxMemoryIncrease });
          
          const endMemory = process.memoryUsage().heapUsed;
          const memoryIncrease = endMemory - startMemory;
          
          results.push({
            operation: op.name,
            memoryIncrease,
            withinLimit: memoryIncrease < maxMemoryIncrease,
            limitExceeded: false
          });
        } catch (error) {
          results.push({
            operation: op.name,
            limitExceeded: true,
            error: error.message
          });
        }
        
        // Force garbage collection if available
        if (global.gc) {
          global.gc();
        }
      }
      
      return results;
    }
  );

  memoryUsageLimits.forEach(result => {
    t.ok(result.withinLimit || result.limitExceeded, 
      `Memory limits enforced for ${result.operation}`);
  });

  // Test 3: CPU time limits
  const cpuTimeLimits = await performanceTracker.measureAsync(
    'cpu-time-limits',
    async () => {
      const maxCPUTime = 5000; // 5 seconds
      
      const cpuIntensiveOps = [
        {
          name: 'complex-xpath',
          xml: generateComplexXML(1000),
          xpath: '//Item[position() mod 2 = 0 and @id > 500]'
        },
        {
          name: 'regex-validation',
          xml: '<Invoice><Email>' + 'a'.repeat(10000) + '@example.com</Email></Invoice>',
          pattern: /^([a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}){1,100}$/
        },
        {
          name: 'recursive-calculation',
          xml: generateNestedCalculations(100)
        }
      ];
      
      const results = [];
      
      for (const op of cpuIntensiveOps) {
        const startTime = Date.now();
        const startCPU = process.cpuUsage();
        
        try {
          const result = await einvoice.processWithTimeout(op, maxCPUTime);
          
          const endTime = Date.now();
          const endCPU = process.cpuUsage(startCPU);
          
          const wallTime = endTime - startTime;
          const cpuTime = (endCPU.user + endCPU.system) / 1000; // Convert to ms
          
          results.push({
            operation: op.name,
            wallTime,
            cpuTime,
            withinLimit: wallTime < maxCPUTime,
            completed: true
          });
        } catch (error) {
          results.push({
            operation: op.name,
            completed: false,
            timeout: error.message.includes('timeout'),
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  cpuTimeLimits.forEach(result => {
    t.ok(result.withinLimit || result.timeout, 
      `CPU time limits enforced for ${result.operation}`);
  });

  // Test 4: Concurrent request limits
  const concurrentRequestLimits = await performanceTracker.measureAsync(
    'concurrent-request-limits',
    async () => {
      const maxConcurrent = 10;
      const totalRequests = 50;
      
      let activeRequests = 0;
      let maxActiveRequests = 0;
      let rejected = 0;
      let completed = 0;
      
      const makeRequest = async (id: number) => {
        try {
          activeRequests++;
          maxActiveRequests = Math.max(maxActiveRequests, activeRequests);
          
          const result = await einvoice.processWithConcurrencyLimit(
            `<Invoice><ID>REQ-${id}</ID></Invoice>`,
            { maxConcurrent }
          );
          
          completed++;
          return { id, success: true };
        } catch (error) {
          if (error.message.includes('concurrent')) {
            rejected++;
          }
          return { id, success: false, error: error.message };
        } finally {
          activeRequests--;
        }
      };
      
      // Launch all requests concurrently
      const promises = [];
      for (let i = 0; i < totalRequests; i++) {
        promises.push(makeRequest(i));
      }
      
      const results = await Promise.all(promises);
      
      return {
        totalRequests,
        completed,
        rejected,
        maxActiveRequests,
        maxConcurrentRespected: maxActiveRequests <= maxConcurrent,
        successRate: completed / totalRequests
      };
    }
  );

  t.ok(concurrentRequestLimits.maxConcurrentRespected, 
    'Concurrent request limit was respected');
  t.ok(concurrentRequestLimits.rejected > 0, 
    'Excess concurrent requests were rejected');

  // Test 5: Rate limiting
  const rateLimiting = await performanceTracker.measureAsync(
    'rate-limiting',
    async () => {
      const rateLimit = 10; // 10 requests per second
      const testDuration = 3000; // 3 seconds
      const expectedMax = (rateLimit * testDuration / 1000) + 2; // Allow small buffer
      
      let processed = 0;
      let rejected = 0;
      const startTime = Date.now();
      
      while (Date.now() - startTime < testDuration) {
        try {
          await einvoice.processWithRateLimit(
            '<Invoice><ID>RATE-TEST</ID></Invoice>',
            { requestsPerSecond: rateLimit }
          );
          processed++;
        } catch (error) {
          if (error.message.includes('rate limit')) {
            rejected++;
          }
        }
        
        // Small delay to prevent tight loop
        await new Promise(resolve => setTimeout(resolve, 10));
      }
      
      const actualRate = processed / (testDuration / 1000);
      
      return {
        processed,
        rejected,
        duration: testDuration,
        actualRate,
        targetRate: rateLimit,
        withinLimit: processed <= expectedMax
      };
    }
  );

  t.ok(rateLimiting.withinLimit, 'Rate limiting is enforced');
  t.ok(rateLimiting.rejected > 0, 'Excess requests were rate limited');

  // Test 6: Nested entity limits
  const nestedEntityLimits = await performanceTracker.measureAsync(
    'nested-entity-limits',
    async () => {
      const entityDepths = [10, 50, 100, 500, 1000];
      const maxDepth = 100;
      
      const results = [];
      
      for (const depth of entityDepths) {
        // Create nested entities
        let entityDef = '<!DOCTYPE foo [\n';
        let entityValue = 'base';
        
        for (let i = 0; i < depth; i++) {
          entityDef += `  <!ENTITY level${i} "${entityValue}">\n`;
          entityValue = `&level${i};`;
        }
        
        entityDef += ']>';
        
        const xml = `<?xml version="1.0" encoding="UTF-8"?>
${entityDef}
<Invoice>
  <Data>${entityValue}</Data>
</Invoice>`;
        
        try {
          await einvoice.parseXML(xml, { maxEntityDepth: maxDepth });
          
          results.push({
            depth,
            allowed: true,
            withinLimit: depth <= maxDepth
          });
        } catch (error) {
          results.push({
            depth,
            allowed: false,
            withinLimit: depth <= maxDepth,
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  nestedEntityLimits.forEach(result => {
    if (result.withinLimit) {
      t.ok(result.allowed, `Entity depth ${result.depth} should be allowed`);
    } else {
      t.notOk(result.allowed, `Entity depth ${result.depth} should be rejected`);
    }
  });

  // Test 7: Output size limits
  const outputSizeLimits = await performanceTracker.measureAsync(
    'output-size-limits',
    async () => {
      const testCases = [
        {
          name: 'normal-output',
          itemCount: 100,
          shouldPass: true
        },
        {
          name: 'large-output',
          itemCount: 10000,
          shouldPass: true
        },
        {
          name: 'excessive-output',
          itemCount: 1000000,
          shouldPass: false
        }
      ];
      
      const maxOutputSize = 100 * 1024 * 1024; // 100MB
      const results = [];
      
      for (const test of testCases) {
        const invoice = {
          id: 'OUTPUT-TEST',
          items: Array(test.itemCount).fill(null).map((_, i) => ({
            id: `ITEM-${i}`,
            description: 'Test item with some description text',
            amount: Math.random() * 1000
          }))
        };
        
        try {
          const output = await einvoice.convertToXML(invoice, {
            maxOutputSize
          });
          
          results.push({
            name: test.name,
            itemCount: test.itemCount,
            outputSize: output.length,
            passed: true,
            expectedPass: test.shouldPass
          });
        } catch (error) {
          results.push({
            name: test.name,
            itemCount: test.itemCount,
            passed: false,
            expectedPass: test.shouldPass,
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  outputSizeLimits.forEach(result => {
    if (result.expectedPass) {
      t.ok(result.passed, `Output ${result.name} should be allowed`);
    } else {
      t.notOk(result.passed, `Output ${result.name} should be limited`);
    }
  });

  // Test 8: Timeout enforcement
  const timeoutEnforcement = await performanceTracker.measureAsync(
    'timeout-enforcement',
    async () => {
      const timeoutTests = [
        {
          name: 'quick-operation',
          delay: 100,
          timeout: 1000,
          shouldComplete: true
        },
        {
          name: 'slow-operation',
          delay: 2000,
          timeout: 1000,
          shouldComplete: false
        },
        {
          name: 'infinite-loop-protection',
          delay: Infinity,
          timeout: 500,
          shouldComplete: false
        }
      ];
      
      const results = [];
      
      for (const test of timeoutTests) {
        const startTime = Date.now();
        
        try {
          await einvoice.processWithTimeout(async () => {
            if (test.delay === Infinity) {
              // Simulate infinite loop
              while (true) {
                // Busy wait
              }
            } else {
              await new Promise(resolve => setTimeout(resolve, test.delay));
            }
            return 'completed';
          }, test.timeout);
          
          const duration = Date.now() - startTime;
          
          results.push({
            name: test.name,
            completed: true,
            duration,
            withinTimeout: duration < test.timeout + 100 // Small buffer
          });
        } catch (error) {
          const duration = Date.now() - startTime;
          
          results.push({
            name: test.name,
            completed: false,
            duration,
            timedOut: error.message.includes('timeout'),
            expectedTimeout: !test.shouldComplete
          });
        }
      }
      
      return results;
    }
  );

  timeoutEnforcement.forEach(result => {
    if (result.expectedTimeout !== undefined) {
      t.equal(result.timedOut, result.expectedTimeout, 
        `Timeout enforcement for ${result.name}`);
    }
  });

  // Test 9: Connection pool limits
  const connectionPoolLimits = await performanceTracker.measureAsync(
    'connection-pool-limits',
    async () => {
      const maxConnections = 5;
      const totalRequests = 20;
      
      const connectionStats = {
        created: 0,
        reused: 0,
        rejected: 0,
        activeConnections: new Set()
      };
      
      const requests = [];
      
      for (let i = 0; i < totalRequests; i++) {
        const request = einvoice.fetchWithConnectionPool(
          `https://example.com/invoice/${i}`,
          {
            maxConnections,
            onConnect: (id) => {
              connectionStats.created++;
              connectionStats.activeConnections.add(id);
            },
            onReuse: () => {
              connectionStats.reused++;
            },
            onReject: () => {
              connectionStats.rejected++;
            },
            onClose: (id) => {
              connectionStats.activeConnections.delete(id);
            }
          }
        ).catch(error => ({ error: error.message }));
        
        requests.push(request);
      }
      
      await Promise.all(requests);
      
      return {
        maxConnections,
        totalRequests,
        connectionsCreated: connectionStats.created,
        connectionsReused: connectionStats.reused,
        requestsRejected: connectionStats.rejected,
        maxActiveReached: connectionStats.created <= maxConnections
      };
    }
  );

  t.ok(connectionPoolLimits.maxActiveReached, 
    'Connection pool limit was respected');

  // Test 10: Resource cleanup verification
  const resourceCleanup = await performanceTracker.measureAsync(
    'resource-cleanup-verification',
    async () => {
      const initialResources = {
        memory: process.memoryUsage(),
        handles: process._getActiveHandles?.()?.length || 0,
        requests: process._getActiveRequests?.()?.length || 0
      };
      
      // Perform various operations that consume resources
      const operations = [
        () => einvoice.parseXML('<Invoice>' + 'A'.repeat(1000000) + '</Invoice>'),
        () => einvoice.validateSchema('<Invoice></Invoice>'),
        () => einvoice.convertFormat({ id: 'TEST' }, 'ubl'),
        () => einvoice.processLargeFile('test.xml', { streaming: true })
      ];
      
      // Execute operations
      for (const op of operations) {
        try {
          await op();
        } catch (error) {
          // Expected for some operations
        }
      }
      
      // Force cleanup
      await einvoice.cleanup();
      
      // Force GC if available
      if (global.gc) {
        global.gc();
        await new Promise(resolve => setTimeout(resolve, 100));
      }
      
      const finalResources = {
        memory: process.memoryUsage(),
        handles: process._getActiveHandles?.()?.length || 0,
        requests: process._getActiveRequests?.()?.length || 0
      };
      
      const memoryLeaked = finalResources.memory.heapUsed - initialResources.memory.heapUsed > 10 * 1024 * 1024; // 10MB threshold
      const handlesLeaked = finalResources.handles > initialResources.handles + 2; // Allow small variance
      const requestsLeaked = finalResources.requests > initialResources.requests;
      
      return {
        memoryBefore: initialResources.memory.heapUsed,
        memoryAfter: finalResources.memory.heapUsed,
        memoryDiff: finalResources.memory.heapUsed - initialResources.memory.heapUsed,
        handlesBefore: initialResources.handles,
        handlesAfter: finalResources.handles,
        requestsBefore: initialResources.requests,
        requestsAfter: finalResources.requests,
        properCleanup: !memoryLeaked && !handlesLeaked && !requestsLeaked
      };
    }
  );

  t.ok(resourceCleanup.properCleanup, 'Resources were properly cleaned up');

  // Print performance summary
  performanceTracker.printSummary();
});

// Helper function to generate complex XML
function generateComplexXML(itemCount: number): string {
  let xml = '<?xml version="1.0" encoding="UTF-8"?><Invoice><Items>';
  
  for (let i = 0; i < itemCount; i++) {
    xml += `<Item id="${i}" category="cat${i % 10}" price="${Math.random() * 1000}">
      <Name>Item ${i}</Name>
      <Description>Description for item ${i}</Description>
    </Item>`;
  }
  
  xml += '</Items></Invoice>';
  return xml;
}

// Helper function to generate nested calculations
function generateNestedCalculations(depth: number): string {
  let xml = '<?xml version="1.0" encoding="UTF-8"?><Invoice>';
  
  for (let i = 0; i < depth; i++) {
    xml += `<Calculation level="${i}">
      <Value>${Math.random() * 100}</Value>
      <Operation>multiply</Operation>`;
  }
  
  xml += '<Result>1</Result>';
  
  for (let i = depth - 1; i >= 0; i--) {
    xml += '</Calculation>';
  }
  
  xml += '</Invoice>';
  return xml;
}

// Run the test
tap.start();