import { tap } from '@git.zone/tstest/tapbundle';
import * as plugins from '../plugins.js';
import { EInvoice } from '../../../ts/index.js';
import { PerformanceTracker } from '../performance.tracker.js';

const performanceTracker = new PerformanceTracker('SEC-07: Schema Validation Security');

tap.test('SEC-07: Schema Validation Security - should securely handle schema validation', async (t) => {
  const einvoice = new EInvoice();

  // Test 1: Malicious schema location
  const maliciousSchemaLocation = await performanceTracker.measureAsync(
    'malicious-schema-location',
    async () => {
      const maliciousXML = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice 
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://malicious.com/steal-data.xsd">
  <ID>TEST-001</ID>
</Invoice>`;
      
      try {
        const result = await einvoice.validateWithSchema(maliciousXML);
        
        return {
          blocked: !result?.valid || result?.schemaBlocked,
          schemaURL: 'http://malicious.com/steal-data.xsd',
          message: 'External schema should be blocked'
        };
      } catch (error) {
        return {
          blocked: true,
          error: error.message
        };
      }
    }
  );

  t.ok(maliciousSchemaLocation.blocked, 'Malicious schema location was blocked');

  // Test 2: Schema with external entity references
  const schemaWithExternalEntities = await performanceTracker.measureAsync(
    'schema-external-entities',
    async () => {
      const xmlWithExternalSchema = `<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE schema [
  <!ENTITY xxe SYSTEM "file:///etc/passwd">
]>
<Invoice xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:noNamespaceSchemaLocation="invoice.xsd">
  <ID>&xxe;</ID>
</Invoice>`;
      
      try {
        const result = await einvoice.validateWithSchema(xmlWithExternalSchema);
        
        return {
          blocked: !result?.valid || !result?.content?.includes('root:'),
          hasXXE: result?.content?.includes('root:') || false
        };
      } catch (error) {
        return {
          blocked: true,
          error: error.message
        };
      }
    }
  );

  t.ok(schemaWithExternalEntities.blocked, 'Schema with external entities was blocked');
  t.notOk(schemaWithExternalEntities.hasXXE, 'XXE content was not resolved');

  // Test 3: Recursive schema imports
  const recursiveSchemaImports = await performanceTracker.measureAsync(
    'recursive-schema-imports',
    async () => {
      const xmlWithRecursiveSchema = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="schema1.xsd">
  <!-- schema1.xsd imports schema2.xsd which imports schema1.xsd -->
  <ID>TEST-001</ID>
</Invoice>`;
      
      const startTime = Date.now();
      const maxTime = 5000; // 5 seconds max
      
      try {
        const result = await einvoice.validateWithSchema(xmlWithRecursiveSchema);
        const timeTaken = Date.now() - startTime;
        
        return {
          prevented: timeTaken < maxTime,
          timeTaken,
          valid: result?.valid || false
        };
      } catch (error) {
        return {
          prevented: true,
          error: error.message
        };
      }
    }
  );

  t.ok(recursiveSchemaImports.prevented, 'Recursive schema imports were prevented');

  // Test 4: Schema complexity attacks
  const schemaComplexityAttack = await performanceTracker.measureAsync(
    'schema-complexity-attack',
    async () => {
      // Create XML with complex nested structure that exploits schema validation
      let complexContent = '<Items>';
      for (let i = 0; i < 1000; i++) {
        complexContent += '<Item>';
        for (let j = 0; j < 100; j++) {
          complexContent += `<SubItem${j} attr1="val" attr2="val" attr3="val"/>`;
        }
        complexContent += '</Item>';
      }
      complexContent += '</Items>';
      
      const complexXML = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  ${complexContent}
</Invoice>`;
      
      const startTime = Date.now();
      const startMemory = process.memoryUsage();
      
      try {
        await einvoice.validateWithSchema(complexXML);
        
        const endTime = Date.now();
        const endMemory = process.memoryUsage();
        
        const timeTaken = endTime - startTime;
        const memoryIncrease = endMemory.heapUsed - startMemory.heapUsed;
        
        return {
          prevented: timeTaken < 10000 && memoryIncrease < 100 * 1024 * 1024,
          timeTaken,
          memoryIncrease
        };
      } catch (error) {
        return {
          prevented: true,
          error: error.message
        };
      }
    }
  );

  t.ok(schemaComplexityAttack.prevented, 'Schema complexity attack was prevented');

  // Test 5: Schema with malicious regular expressions
  const maliciousRegexSchema = await performanceTracker.measureAsync(
    'malicious-regex-schema',
    async () => {
      // XML that would trigger ReDoS if schema uses vulnerable regex
      const maliciousInput = 'a'.repeat(100) + '!';
      const xmlWithMaliciousContent = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice>
  <Email>${maliciousInput}@example.com</Email>
  <Phone>${maliciousInput}</Phone>
</Invoice>`;
      
      const startTime = Date.now();
      
      try {
        await einvoice.validateWithSchema(xmlWithMaliciousContent);
        
        const timeTaken = Date.now() - startTime;
        
        return {
          prevented: timeTaken < 1000, // Should complete quickly
          timeTaken,
          inputLength: maliciousInput.length
        };
      } catch (error) {
        return {
          prevented: true,
          error: error.message
        };
      }
    }
  );

  t.ok(maliciousRegexSchema.prevented, 'Malicious regex in schema was handled safely');

  // Test 6: Schema URL injection
  const schemaURLInjection = await performanceTracker.measureAsync(
    'schema-url-injection',
    async () => {
      const injectionAttempts = [
        'http://example.com/schema.xsd?file=/etc/passwd',
        'http://example.com/schema.xsd#../../admin/schema.xsd',
        'http://example.com/schema.xsd%00.malicious',
        'javascript:alert("XSS")',
        'file:///etc/passwd'
      ];
      
      const results = [];
      
      for (const schemaURL of injectionAttempts) {
        const xml = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="${schemaURL}">
  <ID>TEST</ID>
</Invoice>`;
        
        try {
          const result = await einvoice.validateWithSchema(xml);
          results.push({
            url: schemaURL,
            blocked: !result?.valid || result?.schemaBlocked,
            allowed: result?.valid && !result?.schemaBlocked
          });
        } catch (error) {
          results.push({
            url: schemaURL,
            blocked: true,
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  schemaURLInjection.forEach(result => {
    t.ok(result.blocked, `Schema URL injection blocked: ${result.url}`);
  });

  // Test 7: Schema include/import security
  const schemaIncludeSecurity = await performanceTracker.measureAsync(
    'schema-include-security',
    async () => {
      // Test schema that tries to include external resources
      const xmlWithIncludes = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <!-- Schema tries to include external files -->
  <ID>TEST-001</ID>
</Invoice>`;
      
      const testCases = [
        { type: 'local-file', path: '../../../etc/passwd' },
        { type: 'remote-url', path: 'http://evil.com/malicious.xsd' },
        { type: 'relative-path', path: '../../../../sensitive/data.xsd' }
      ];
      
      const results = [];
      
      for (const testCase of testCases) {
        try {
          const result = await einvoice.validateSchemaIncludes(xmlWithIncludes, testCase.path);
          results.push({
            type: testCase.type,
            blocked: !result?.allowed,
            path: testCase.path
          });
        } catch (error) {
          results.push({
            type: testCase.type,
            blocked: true,
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  schemaIncludeSecurity.forEach(result => {
    t.ok(result.blocked, `Schema include blocked: ${result.type}`);
  });

  // Test 8: Schema validation bypass attempts
  const schemaBypassAttempts = await performanceTracker.measureAsync(
    'schema-validation-bypass',
    async () => {
      const bypassAttempts = [
        {
          name: 'namespace-confusion',
          xml: `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xmlns="fake-namespace" xmlns:real="actual-namespace">
  <ID>BYPASS-001</ID>
  <real:MaliciousData>attack-payload</real:MaliciousData>
</Invoice>`
        },
        {
          name: 'schema-version-mismatch',
          xml: `<?xml version="1.0" encoding="UTF-8"?>
<Invoice version="99.99">
  <ID>BYPASS-002</ID>
  <UnsupportedElement>should-not-validate</UnsupportedElement>
</Invoice>`
        },
        {
          name: 'encoding-trick',
          xml: `<?xml version="1.0" encoding="UTF-16"?>
<Invoice>
  <ID>BYPASS-003</ID>
  <HiddenData>malicious</HiddenData>
</Invoice>`
        }
      ];
      
      const results = [];
      
      for (const attempt of bypassAttempts) {
        try {
          const result = await einvoice.validateWithSchema(attempt.xml);
          results.push({
            name: attempt.name,
            valid: result?.valid || false,
            caught: !result?.valid || result?.hasWarnings
          });
        } catch (error) {
          results.push({
            name: attempt.name,
            caught: true,
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  schemaBypassAttempts.forEach(result => {
    t.ok(result.caught, `Schema bypass attempt caught: ${result.name}`);
  });

  // Test 9: Schema caching security
  const schemaCachingSecurity = await performanceTracker.measureAsync(
    'schema-caching-security',
    async () => {
      const results = {
        cachePoison: false,
        cacheBypass: false,
        cacheOverflow: false
      };
      
      // Test 1: Cache poisoning
      try {
        // First, load legitimate schema
        await einvoice.loadSchema('legitimate.xsd');
        
        // Try to poison cache with malicious version
        await einvoice.loadSchema('legitimate.xsd', { 
          content: '<malicious>content</malicious>',
          forceReload: false 
        });
        
        // Check if cache was poisoned
        const cachedSchema = await einvoice.getSchemaFromCache('legitimate.xsd');
        results.cachePoison = cachedSchema?.includes('malicious') || false;
      } catch (error) {
        // Error is good - means poisoning was prevented
      }
      
      // Test 2: Cache bypass
      try {
        const xml = `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xsi:schemaLocation="cached-schema.xsd?nocache=${Date.now()}">
  <ID>TEST</ID>
</Invoice>`;
        
        const result1 = await einvoice.validateWithSchema(xml);
        const result2 = await einvoice.validateWithSchema(xml);
        
        // Should use cache, not fetch twice
        results.cacheBypass = result1?.cacheHit === false && result2?.cacheHit === true;
      } catch (error) {
        // Expected
      }
      
      // Test 3: Cache overflow
      try {
        // Try to overflow cache with many schemas
        for (let i = 0; i < 10000; i++) {
          await einvoice.loadSchema(`schema-${i}.xsd`);
        }
        
        // Check memory usage
        const memUsage = process.memoryUsage();
        results.cacheOverflow = memUsage.heapUsed > 500 * 1024 * 1024; // 500MB
      } catch (error) {
        // Expected - cache should have limits
      }
      
      return results;
    }
  );

  t.notOk(schemaCachingSecurity.cachePoison, 'Cache poisoning was prevented');
  t.notOk(schemaCachingSecurity.cacheOverflow, 'Cache overflow was prevented');

  // Test 10: Real-world schema validation
  const realWorldSchemaValidation = await performanceTracker.measureAsync(
    'real-world-schema-validation',
    async () => {
      const formats = ['ubl', 'cii', 'zugferd'];
      const results = [];
      
      for (const format of formats) {
        try {
          // Create a valid invoice for the format
          const invoice = createTestInvoice(format);
          
          // Validate with proper schema
          const validationResult = await einvoice.validateWithSchema(invoice, {
            format,
            strict: true,
            securityChecks: true
          });
          
          results.push({
            format,
            valid: validationResult?.valid || false,
            secure: validationResult?.securityPassed || false,
            errors: validationResult?.errors || []
          });
        } catch (error) {
          results.push({
            format,
            valid: false,
            secure: false,
            error: error.message
          });
        }
      }
      
      return results;
    }
  );

  realWorldSchemaValidation.forEach(result => {
    t.ok(result.secure, `${result.format} schema validation is secure`);
  });

  // Print performance summary
  performanceTracker.printSummary();
});

// Helper function to create test invoices
function createTestInvoice(format: string): string {
  const invoices = {
    ubl: `<?xml version="1.0" encoding="UTF-8"?>
<Invoice xmlns="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2">
  <UBLVersionID>2.1</UBLVersionID>
  <ID>INV-001</ID>
  <IssueDate>2024-01-15</IssueDate>
</Invoice>`,
    
    cii: `<?xml version="1.0" encoding="UTF-8"?>
<rsm:CrossIndustryInvoice xmlns:rsm="urn:un:unece:uncefact:data:standard:CrossIndustryInvoice:100">
  <rsm:ExchangedDocument>
    <ram:ID>INV-001</ram:ID>
  </rsm:ExchangedDocument>
</rsm:CrossIndustryInvoice>`,
    
    zugferd: `<?xml version="1.0" encoding="UTF-8"?>
<rsm:CrossIndustryInvoice xmlns:rsm="urn:un:unece:uncefact:data:standard:CrossIndustryInvoice:100">
  <rsm:ExchangedDocumentContext>
    <ram:GuidelineSpecifiedDocumentContextParameter>
      <ram:ID>urn:cen.eu:en16931:2017:compliant:factur-x.eu:1p0:basic</ram:ID>
    </ram:GuidelineSpecifiedDocumentContextParameter>
  </rsm:ExchangedDocumentContext>
</rsm:CrossIndustryInvoice>`
  };
  
  return invoices[format] || invoices.ubl;
}

// Run the test
tap.start();